Extension { #name : 'RwProjectReferenceDefinition' }

{ #category : '*rowan-gemstone-components-extensions' }
RwProjectReferenceDefinition >> asSpecification [

	| repoSpec platformSpec platformDict |
	self useGit
		ifTrue: [
			repoSpec := RwGitRepositorySpecification new
				committish: self committish;
				committishType: self committishType;
				remoteUrl: self remoteUrl;
				yourself ]
		ifFalse: [  repoSpec := RwDiskRepositorySpecification new ] .
	platformSpec := RwGemStoneSpecification new.
	platformSpec packageNameToPlatformPropertiesMap: self packageNameToPlatformPropertiesMap.
	platformDict := (Dictionary new) at: 'gemstone' put: platformSpec; yourself.
	^ RwComponentSpecification new
		specName: self projectName;
		projectUrl: self projectUrl;
		repoPath: self packagesPath;
		configsPath: self configsPath;
		specsPath: self specsPath;
		projectsPath: self projectsPath;
		defaultComponentName: self defaultComponentName;
		defaultConfigurationNames: self configurationNames;
		defaultGroupNames: self groupNames;
		comment: self comment;
		repoSpec: repoSpec;
		platformSpec: platformDict;
		yourself.
]

{ #category : '*rowan-gemstone-components-extensions' }
RwProjectReferenceDefinition >> defaultMethodEnv [

	^ self properties at: 'defaultMethodEnv' ifAbsent: [ 0 ]
]

{ #category : '*rowan-gemstone-components-extensions' }
RwProjectReferenceDefinition >> defaultMethodEnv: anInteger [

	^ self properties at: 'defaultMethodEnv' put: anInteger
]

{ #category : '*rowan-gemstone-components-extensions' }
RwProjectReferenceDefinition >> defaultSymbolDictName [

	^ self properties at: 'defaultSymbolDictName' ifAbsent: [ 'UserGlobals' ]
]

{ #category : '*rowan-gemstone-components-extensions' }
RwProjectReferenceDefinition >> defaultSymbolDictName: aString [

	^ self properties at: 'defaultSymbolDictName' put: aString
]

{ #category : '*rowan-gemstone-components-extensions' }
RwProjectReferenceDefinition >> defaultUseSessionMethodsForExtensions [

	^ self properties at: 'defaultUseSessionMethodsForExtensions' ifAbsent: [ false ]
]

{ #category : '*rowan-gemstone-components-extensions' }
RwProjectReferenceDefinition >> defaultUseSessionMethodsForExtensions: aBool [

	^ self properties at: 'defaultUseSessionMethodsForExtensions' put: aBool
]

{ #category : '*rowan-gemstone-components-extensions' }
RwProjectReferenceDefinition >> methodEnvForPackageNamed: packageName [

	| packageProperties |
	packageProperties :=self packageNameToPlatformPropertiesMap
		at: packageName
		ifAbsent: [ ^ self defaultMethodEnv ].
	^ packageProperties at: 'methodEnv' ifAbsent: [ self defaultMethodEnv ]
]

{ #category : '*rowan-gemstone-components-extensions' }
RwProjectReferenceDefinition >> packageNameToPlatformPropertiesMap [

	^ self properties at: 'packageNameToPlatformPropertiesMap' ifAbsentPut: [ Dictionary new ]
]

{ #category : '*rowan-gemstone-components-extensions' }
RwProjectReferenceDefinition >> packageNameToPlatformPropertiesMap: aDictionary [

	^ self properties at: 'packageNameToPlatformPropertiesMap' put: aDictionary
]

{ #category : '*rowan-gemstone-components-extensions' }
RwProjectReferenceDefinition >> setSymbolDictName: symbolDictName forPackageNamed: packageName [

	| packageProperties packageNameToPlatformPropertiesMap |
	packageNameToPlatformPropertiesMap := self packageNameToPlatformPropertiesMap.
	packageProperties := packageNameToPlatformPropertiesMap
		at: packageName
		ifAbsent: [ packageNameToPlatformPropertiesMap at: packageName put: Dictionary new ].
	symbolDictName 
		ifNil: [
			"remove the entry if it exists"
			packageProperties removeKey: 'symbolDictName' ifAbsent: [].
			^self ].
	packageProperties at: 'symbolDictName' put: symbolDictName
]

{ #category : '*rowan-gemstone-components-extensions' }
RwProjectReferenceDefinition >> setUseSessionMethodsForExtensions: aBool forPackageNamed: packageName [

	| packageProperties |
	packageProperties := self packageNameToPlatformPropertiesMap
		at: packageName
		ifAbsent: [ self packageNameToPlatformPropertiesMap at: packageName put: Dictionary new ].
	aBool 
		ifNil: [
			"remove the entry if it exists"
			packageProperties removeKey: 'useSessionMethodsForExtensions' ifAbsent: [].
			^self ].
	aBool ifTrue: [ Rowan image ensureSessionMethodsEnabled ].
	packageProperties at: 'useSessionMethodsForExtensions' put: aBool
]

{ #category : '*rowan-gemstone-components-extensions' }
RwProjectReferenceDefinition >> symbolDictNameForPackageNamed: packageName [

	| packageProperties |
	packageProperties := self packageNameToPlatformPropertiesMap
		at: packageName
		ifAbsent: [ ^ self defaultSymbolDictName ].
	^ packageProperties
		at: 'symbolDictName'
		ifAbsent: [ ^ self defaultSymbolDictName ]
]

{ #category : '*rowan-gemstone-components-extensions' }
RwProjectReferenceDefinition >> useSessionMethodsForExtensionsForPackageNamed: packageName [

	| packageProperties |
	packageProperties := self packageNameToPlatformPropertiesMap
		at: packageName
		ifAbsent: [ ^ self defaultUseSessionMethodsForExtensions ].
	^ packageProperties
		at: 'useSessionMethodsForExtensions'
		ifAbsent: [ ^ self defaultUseSessionMethodsForExtensions ]
]
