Extension { #name : 'Fraction' }

{ #category : 'Storing and Loading' }
Fraction class >> loadFrom: passiveObj [

"Reads from passiveObj the passive form of an object.  Converts the object to
 its active form by loading the information into a new instance of the receiver.
 Returns the new instance."

| inst num den |

passiveObj readNamedIV.
num := passiveObj ivValue.
passiveObj readNamedIV.
den := passiveObj ivValue.

passiveObj skipNamedInstVars.

inst := self _newFraction: num denominator: den.
passiveObj hasRead: inst.  "arg to hasRead must not be a SmallFraction."
^inst.

]

{ #category : 'Private' }
Fraction >> _numerator: n denom: d [
  numerator := n .
  denominator := d .
  ^ self immediateInvariant

]

{ #category : 'Accessing' }
Fraction >> denominator [

"Returns the denominator of the receiver."

^ denominator

]

{ #category : 'Accessing' }
Fraction >> instVarAt: anIndex put: aValue [

"Disallowed.  You may not change the value of a Fraction."

self shouldNotImplement: #instVarAt:put:

]

{ #category : 'Accessing' }
Fraction >> numerator [

"Returns the numerator of the receiver."

^ numerator

]

{ #category : 'Copying' }
Fraction >> postCopy [
  ^ self immediateInvariant

]

{ #category : 'Storing and Loading' }
Fraction >> writeTo: passiveObj [

"Converts the receiver to its passive form and writes that information on
 passiveObj."

"Reimplemented from Number since the receiver has a non-literal representation."

^super basicWriteTo: passiveObj

]
