"
GsFileIn supports filing in from topaz-format GemStone source files into the image, without the use of topaz.
	
	To use, send one of the from* methods.  This also performs the file in.
	  for example,
	     GsFileIn fromServerPath: 'mySourceCode.gs'
	
	Only a subset of topaz commands is supported:
	   doit , printit , run , nbrun, send
	   input
	   category:
	   classmethod , classmethod:
	   method , method:
	   removeallmethods , removeallclassmethods
	   env N
	   set compile_env N , set class
	     (other set commands are ignored)
Following can be read but are ignored:
	   expectvalue, expecterror, iferr, iferr_clear, iferr_list, errorcount
	   fileformat, fileout, output
	   display, omit, level, limit
	   time, remark, status
"
Class {
	#name : 'GsFileIn',
	#superclass : 'Object',
	#instVars : [
		'session',
		'fileStream',
		'line',
		'lineNum',
		'currentClass',
		'category',
		'compileEnvironment'
	],
	#category : 'Kernel'
}

{ #category : 'other' }
GsFileIn class >> _fromStream: aFileStream [
"file in from a stream to current session"

	^self _fromStream: aFileStream to: nil

]

{ #category : 'other' }
GsFileIn class >> _fromStream: aFileStream to: anExternalSessionOrNil [
"anExternalSessionOrNil can be nil meaning file in will happen in current session"

	^self new
		fileStream: aFileStream;
		setSession: anExternalSessionOrNil;
		yourself

]

{ #category : 'other' }
GsFileIn class >> fromClientPath: aString [
"file in from path on session client machine into current session"

	self
		fromClientPath: aString
		to: nil.

]

{ #category : 'other' }
GsFileIn class >> fromClientPath: aString to: anExternalSession [

	self
		fromPath: aString
		on: #clientText
		to: anExternalSession.

]

{ #category : 'other' }
GsFileIn class >> fromGciHostPath: aString [

	self
		fromGciHostPath: aString
		to: nil.

]

{ #category : 'other' }
GsFileIn class >> fromGciHostPath: aString to: anExternalSession [

	self
		fromPath: aString
		on: #clientText
		to: anExternalSession.

]

{ #category : 'other' }
GsFileIn class >> fromGemHostPath: aString [

	self
		fromGemHostPath: aString
		to: nil.

]

{ #category : 'other' }
GsFileIn class >> fromGemHostPath: aString to: anExternalSession [

	self
		fromPath: aString
		on: #serverText
		to: anExternalSession.

]

{ #category : 'other' }
GsFileIn class >> fromNestedClientPath: aString to: anExternalSession [

	self
		fromNestedPath: aString
		on: #clientText
		to: anExternalSession.
]

{ #category : 'other' }
GsFileIn class >> fromNestedPath: aString on: aFileType to: anExternalSession [
"aString is file path to file containg topaz file-out format of smalltalk code.
type is either #clientFile or #serverFile to denote relative user action to use.
anExternal session is needed if file-in happens into that given session"

	| fileStream  |

	fileStream := FileStreamPortable read: aString type: aFileType.
	[ (self _fromStream: fileStream to: anExternalSession) 
         _clearTopazSessionState: false;
         doFileIn	] ensure: [fileStream close].
]

{ #category : 'other' }
GsFileIn class >> fromNestedServerPath: aString to: anExternalSession [

	self
		fromNestedPath: aString
		on: #serverText
		to: anExternalSession.
]

{ #category : 'other' }
GsFileIn class >> fromPath: aString on: aFileType to: anExternalSession [
"aString is file path to file containg topaz file-out format of smalltalk code.
type is either #clientFile or #serverFile to denote relative user action to use.
anExternal session is needed if file-in happens into that given session"

	| fileStream  |

	fileStream := FileStreamPortable read: aString type: aFileType.
	[ (self _fromStream: fileStream to: anExternalSession) doFileIn	] ensure: [fileStream close].

]

{ #category : 'other' }
GsFileIn class >> fromServerPath: aString [

	self
		fromServerPath: aString
		to: nil.

]

{ #category : 'other' }
GsFileIn class >> fromServerPath: aString to: anExternalSession [

	self
		fromPath: aString
		on: #serverText
		to: anExternalSession.

]

{ #category : 'other' }
GsFileIn class >> new [

	^super new
		initialize;
		yourself.

]

{ #category : 'accessors' }
GsFileIn >> _clearTopazSessionState [

	^ (self dynamicInstVarAt: #'_clearTopazSessionState') isNil
]

{ #category : 'accessors' }
GsFileIn >> _clearTopazSessionState: aBool [

	self dynamicInstVarAt: #'_clearTopazSessionState' put: aBool
]

{ #category : 'private' }
GsFileIn >> _defaultCategory [
	^ 'as yet unspecified'
]

{ #category : 'private' }
GsFileIn >> _sendIsLegal [
  ^ true

]

{ #category : 'processing' }
GsFileIn >> abort [

	^System abortTransaction

]

{ #category : 'processing' }
GsFileIn >> abortTransaction [

	^session abort

]

{ #category : 'processing' }
GsFileIn >> category [
	| cat |
	cat := (line copyFrom: 10 to: line size) trimSeparators.
	self currentCategory: cat
]

{ #category : 'processing' }
GsFileIn >> classMethod [

	currentClass isNil ifTrue: [self error: 'current class not defined'].
	self compileMethodIn: currentClass , ' class'.

]

{ #category : 'processing' }
GsFileIn >> classMethod: aString [

	aString ifNotNil:[ currentClass := aString ].
	self classMethod.

]

{ #category : 'processing' }
GsFileIn >> commit [

	^System commitTransaction

]

{ #category : 'processing' }
GsFileIn >> commitTransaction [

	^session commit

]

{ #category : 'processing' }
GsFileIn >> compileEnvironment: arg [

	arg isDigits ifFalse: [self error: 'ENV  only accepts integers'].
  compileEnvironment := arg .

]

{ #category : 'processing' }
GsFileIn >> compileMethodIn: aClassContex [
"compile method from stream in context of current class"

	| string |
	string := aClassContex , '
compileMethod: ' , self nextChunk printString , '
dictionaries: System myUserProfile symbolList
category: ' , category printString,
' environmentId: ', compileEnvironment.
	self execute: string.

]

{ #category : 'processing' }
GsFileIn >> currentCategory: cat [
  category := cat .
	cat isEmpty ifTrue: [self error: 'category is empty'].
	cat first = $' ifTrue: [
		cat last = $' ifFalse:[
      self error: 'category begins with quote but does not end with quote'
    ].
		category := cat copyFrom: 2 to: category size - 1.
	] ifFalse:[
    cat last = $' ifTrue:[
       self error: 'category ends with quote but does not start with quote'.
    ].
  ].
	(category includes: $') ifTrue:[ category := cat evaluate ].
]

{ #category : 'processing' }
GsFileIn >> currentClass [

	^currentClass

]

{ #category : 'processing' }
GsFileIn >> currentClass: aClassName [
    "The current category is cleared by the LOGOUT, LOGIN, SET CLASS,
    and SET SESSION commands. "

	currentClass:= aClassName.
	category := self _defaultCategory.
]

{ #category : 'processing' }
GsFileIn >> doFileIn [
[  lineNum := 0 .
	session isNil ifTrue: [self session: self].
	[
		fileStream atEnd not.
	] whileTrue: [
		line := fileStream nextLine trimSeparators.
    lineNum := lineNum + 1 .
		[
      self processLine
    ] on: Error do:[:ex | | ln |
      ln := ', at line ', lineNum asString .
      ex messageText ifNil:[ ex messageText: ln ]
                  ifNotNil:[:str | ex messageText:(str copy addAll: ln; yourself)].
      ex pass
    ]
	]
    ] ensure: [ 
      "topmost doFileIn should clear the topaz sesion state"
      self _clearTopazSessionState 
        ifTrue: [ 
          self currentProject: nil. 
          self currentPackage: nil ]
    ]
]

{ #category : 'processing' }
GsFileIn >> doit [

	self execute: self nextChunk.

]

{ #category : 'processing' }
GsFileIn >> execute: aString [

	| string result |
  session == self ifFalse:[
	  string := '[' , aString , '] on: Error do: [:ex | ^ex description]. nil'.
	  (result := session executeString: string) ifNotNil:[:aDescription |
      self error: aDescription .
      ^ Error signal: aDescription "do not expect to be here"
    ].
  ] ifTrue:[
    result := aString evaluate
  ].
  ^ result

]

{ #category : 'processing' }
GsFileIn >> executeString:  string [

	string evaluate.
	^nil

]

{ #category : 'accessors' }
GsFileIn >> fileStream: aFileStream [

	fileStream := aFileStream.

]

{ #category : 'processing' }
GsFileIn >> ignoreList [

	^#('EXPECTVALUE'
		'EXPECTERROR'
		'ERRORCOUNT'
		'FILEOUT'
		'FILEFORMAT'
		'DISPLAY'
		'LEVEL'
		'LIMIT'
		'LIST'
		'IFERR'
		'IFERR_LIST'
		'IFERR_CLEAR'
		'OMIT'
		'OUTPUT'
		'REMARK'
		'STATUS'
		'TIME')

]

{ #category : 'Initialize' }
GsFileIn >> initialize [

	session := self.
	category := self _defaultCategory.
	compileEnvironment := '0' .
]

{ #category : 'processing' }
GsFileIn >> inputNestedFile: aPath [
	"input nested file"

	| aSession isClient |
	session == self
		ifFalse: [ aSession := session ].
	isClient := (fileStream respondsTo: #'gsfile')
		ifTrue: [ fileStream gsfile isClient ]
		ifFalse: [ false ].
	isClient
		ifTrue: [ GsFileIn fromNestedClientPath: aPath to: aSession ]
		ifFalse: [ GsFileIn fromNestedServerPath: aPath to: aSession ]
]

{ #category : 'processing' }
GsFileIn >> method [

	currentClass isNil ifTrue: [self error: 'current class not defined'].
	self compileMethodIn: currentClass.

]

{ #category : 'processing' }
GsFileIn >> method: aString [

	aString ifNotNil:[ currentClass := aString ].
	self method.

]

{ #category : 'processing' }
GsFileIn >> nextChunk [

	| str |
	str := String new.
	[
		fileStream atEnd not.
	] whileTrue: [
		line := fileStream nextLine trimTrailingSeparators.
    lineNum := lineNum + 1 .
    (line notEmpty and: [line first = $%]) ifTrue: [
			^ str
		].
		str addAll: line; lf.
	].
	self error: 'ran off end of file!'.

]

{ #category : 'processing' }
GsFileIn >> processLine [
	| words command firstChar |
	line isEmpty ifTrue: [^self].
	(#('DOIT' 'PRINTIT' 'RUN' 'NBRUN') includes: line asUppercase) ifTrue: [^self doit].
	words := line subStrings.
	command := words first asUppercase.
	((firstChar := command at:1) == $! or:[ firstChar == $#])  ifTrue: [^nil].

	firstChar == $S ifTrue:[
	  command = 'SET' ifTrue: [
		  ((words at: 2) equalsNoCase: 'compile_env:') ifTrue:[
				words size == 3 ifFalse:[
					self error:'wrong number of arguments to SET COMPILE_ENV:'
				].
				^self compileEnvironment: (words at: 3)
			].
		  (((words at: 2) equalsNoCase: 'class')
			  or: [(words at: 2) equalsNoCase: 'class:']) ifTrue: [
				  words size == 3 ifFalse:[
					  self error:'wrong number of arguments to SET CLASS'
				  ].
				  ^self currentClass: (words at: 3)
           ].
		  (((words at: 2) equalsNoCase: 'category')
			  or: [(words at: 2) equalsNoCase: 'category:']) ifTrue: [
				  words size == 3 ifFalse:[
					  self error:'wrong number of arguments to SET CATEGORY'
				  ].
				  ^self currentCategory: (words at: 3)
			  ].
		  ((words at: 2) equalsNoCase: 'enableremoveall') ifTrue: [
				  words size == 3 ifFalse:[
					  self error:'wrong number of arguments to SET ENABLEREMOVEALL'
				  ].
				  ^self setEnableRemoveAll: ((words at: 3) equalsNoCase: 'on')
			  ].
		  (((words at: 2) equalsNoCase: 'project')
			  or: [(words at: 2) equalsNoCase: 'project:']) ifTrue: [
                  words size == 2 ifTrue: [ 
                      "clear current project"
                      ^self currentProject: nil ].
				  words size == 3 ifFalse:[
					  self error:'wrong number of arguments to SET PROJECT'
				  ].
				  ^self currentProject: (words at: 3)
			  ].
		  (((words at: 2) equalsNoCase: 'package')
			  or: [(words at: 2) equalsNoCase: 'package:']) ifTrue: [
                  words size == 2 ifTrue: [ 
                      "clear current package"
                      ^self currentPackage: nil ].
				  words size == 3 ifFalse:[
					  self error:'wrong number of arguments to SET PACKAGE'
				  ].
				  ^self currentPackage: (words at: 3)
			  ].
      #( 'comp' 'cla' 'cat' 'enable' 'proj' 'pack' ) do:[:str |
        ((words at: 2) at: 1 equals: str) ifTrue:[
           self error:'unrecognized SET ', (words at: 2) asString .
        ].
      ].
      GsFile gciLogServer: 'WARNING, unrecognized SET ', (words at: 2) asString.
		  ^nil
    ].
    command = 'SEND' ifTrue:[
      self _sendIsLegal ifFalse:[ self error: 'SEND not supported' ].
      words size > 3 ifTrue:[ self error:'more than 2 args to SEND'].
      ^ self execute: (words at: 2), ' ', (words at: 3)
    ]
  ].
  firstChar == $C ifTrue:[
	  (command = 'CATEGORY:' or: [command = 'CATEGORY']) ifTrue: [^self category].
	  command = 'CLASSMETHOD' ifTrue: [^self classMethod].
	  command = 'CLASSMETHOD:' ifTrue: [^self classMethod: (words atOrNil: 2)].
	  command = 'COMMIT' ifTrue: [^self commitTransaction].
  ].
  firstChar == $M ifTrue:[
	  command = 'METHOD' ifTrue: [^self method].
	  command = 'METHOD:' ifTrue: [^self method: (words atOrNil: 2)].
  ].
  firstChar == $E ifTrue:[
    command = 'ENV' ifTrue:[
      words size == 2 ifTrue:[ ^ self compileEnvironment: (words at: 2)].
      words size == 1 ifTrue:[ ^ nil "no change to env"].
      self error:'wrong number of arguments to ENV'.
    ].
   ].
	firstChar == $R ifTrue: [
		command = 'REMOVEALLMETHODS' ifTrue: [^self removeAllMethods: (words atOrNil: 2)].
		command = 'REMOVEALLCLASSMETHODS' ifTrue: [^self removeAllClassMethods: (words atOrNil: 2)].
	].
	firstChar == $A ifTrue: [
		command = 'ABORT' ifTrue: [^self abortTransaction].
	].
	firstChar == $I ifTrue: [
		command = 'INPUT' ifTrue: [words size == 1 ifTrue: [self error:'wrong number of arguments to input'].
		^self inputNestedFile: (words at: 2)].
	].
  ^(self ignoreList includes: command)
		ifTrue: [nil ]
		ifFalse: [self error:  'unrecognized command: ' , command printString ].
]

{ #category : 'processing' }
GsFileIn >> removeAllClassMethods [

	self removeAllEnabled ifFalse: [ ^ self ].
	currentClass isNil ifTrue: [self error: 'current class not defined'].
	self execute: currentClass ,' class removeAllMethods'.
]

{ #category : 'processing' }
GsFileIn >> removeAllClassMethods: aClassName [
"removes all  class methods for supplied class. Supplied class becomes current class"

	self removeAllEnabled ifFalse: [ ^ self ].
	aClassName notNil ifTrue: [currentClass := aClassName].
	currentClass isNil ifTrue: [self error: 'current class not defined'].
	self removeAllClassMethods
]

{ #category : 'accessors' }
GsFileIn >> removeAllEnabled [
	"if true (default), removeAllMethods is enabled"

	^ (self dynamicInstVarAt: #'enableRemoveAll') ~~ false
]

{ #category : 'processing' }
GsFileIn >> removeAllMethods [

	self removeAllEnabled ifFalse: [ ^ self ].
	currentClass isNil ifTrue: [self error: 'current class not defined'].
	self execute: currentClass , ' removeAllMethods'.
]

{ #category : 'processing' }
GsFileIn >> removeAllMethods: aClassName [
"remove all methods for supplied class. Supplied class becomes current class"

	self removeAllEnabled ifFalse: [ ^ self ].
	aClassName notNil ifTrue: [currentClass := aClassName].
	currentClass isNil ifTrue: [self error: 'current class not defined'].
	self removeAllMethods
]

{ #category : 'accessors' }
GsFileIn >> session: anExternalSession [
    "The current category is cleared by the LOGOUT, LOGIN, SET CLASS,
    and SET SESSION commands. "

	session := anExternalSession.
	category := self _defaultCategory.
]

{ #category : 'accessors' }
GsFileIn >> setCurrentClass: aClassName [
    "The current category is cleared by the LOGOUT, LOGIN, SET CLASS,
    and SET SESSION commands. "

	self currentClass: aClassName
]

{ #category : 'accessors' }
GsFileIn >> setEnableRemoveAll: aBool [
	"if true (default), removeAllMethods is enabled"

	self dynamicInstVarAt: #'enableRemoveAll' put: aBool
]

{ #category : 'accessors' }
GsFileIn >> setSession: anExternalSession [
    "The current category is cleared by the LOGOUT, LOGIN, SET CLASS,
    and SET SESSION commands. "

	self session: (anExternalSession ifNil: [self])
]
