Extension { #name : 'Object' }

{ #category : 'Object Canonicalization' }
Object >> _canonicalizeReferencesUsingPolicy: aReferencingObjectPolicy [

	^aReferencingObjectPolicy canonicalizeReferencesInObject: self

]

{ #category : 'Repository Conversion' }
Object >> fixReferencesAfterConversionFromDirectory: string [

"Attempt to old objects to special objects as follows:
        SmallFloat ->           SmallDouble
        Float ->                SmallDouble
        ObsLargePositiveInteger ->      SmallInteger
        ObsLargeNegativeInteger ->      SmallInteger

Note: some instances may not be converted for the following reasons:
        Instance is in an IdentitySet or IdentityBag larger than 2024 elements.
        Instances is in an Array or other oop collection larger than 2024 elements.
        Value of the instance is out of range of the new special object class."

| fn aString refs objs2ndPass alreadyFixed |
  refs := GsBitmap newForHiddenSet: #Customer1.
  objs2ndPass := GsBitmap newForHiddenSet: #Customer2.
  alreadyFixed := GsBitmap newForHiddenSet: #Customer5.
  refs removeAll.
  objs2ndPass removeAll.
  alreadyFixed removeAll.

"Load in the 2 bitmaps that contain refs to LargeInts and Floats"
aString := String withAll: string.
(aString last == $/)
        ifFalse:[aString add: '/'].
fn := String new.
fn addAll: aString; addAll: 'AllLrgIntRefs.bm'.
(GsFile existsOnServer: fn)
        ifTrue:[refs readFromFile: fn]
        ifFalse:[self halt: ('Cannot open file ', fn) ].

fn size: 0; addAll: aString; addAll: 'AllFloatRefs.bm'.
"This one is optional because 6.1.5 conversion does not produce AllFloatRefs.bm"
(GsFile existsOnServer: fn)
        ifTrue:[refs readFromFile: fn].

(refs size) == 0
        ifTrue:[^false].

[ | array |
  System transactionMode: #autoBegin.
  System signalAlmostOutOfMemoryThreshold: 70 .

  [ array := refs enumerateWithLimit: 2000 startingAfter: 0.
    array size == 0] whileFalse:[
        1 to: array size do:[:n| |obj|
                obj := array at: n.
                obj fixReferencesInFirstPass
                        ifTrue:[obj fixRefsAfterConversion] "do it now"
                        ifFalse:[objs2ndPass add: obj]. "save it for later"
                refs removeAll: array.
        ].
  ].
  System commitTransaction ifFalse:[ Object halt: 'error: commit failure!'].
  [ array := objs2ndPass enumerateWithLimit: 2000 startingAfter: 0. array size == 0] whileFalse:[
     1 to: array size do:[:n| |obj|
        obj := array at: n.
        obj fixRefsAfterConversion.
        objs2ndPass removeAll: array
     ].
  ].
  System commitTransaction ifFalse:[Object halt: 'error: commit failure!'].
  ^ alreadyFixed size

] onException: AlmostOutOfMemory do:[:ex |
  "do a commit..."
  System commitTransaction
           ifFalse:[Object halt: 'error: commit failure!'].
   "run another markSweep, to see if we can drop below the threshold..."
  System _vmMarkSweep.
  System signalAlmostOutOfMemoryThreshold: 70 .
  System enableAlmostOutOfMemoryError. "re-enable the exception"
  ex resume
].

]

{ #category : 'Repository Conversion' }
Object >> fixRefsAfterConversion [

"Default method for fixing references ObsLargePositiveInteger and
 ObsLargeNegativeInteger instances that can now be represented as
 a SmallInteger and Floats and SmallFloats which can now be represented as
 a SmallDouble."

| convBm |
convBm := (GsBitmap newForHiddenSet: #Conversion).
(convBm includes: self)
        ifTrue:[^false]. "already fixed this one"

self isInternalObject
        ifFalse:[
                self fixInstVarRefsAfterConversion.
                self fixIndexableRefsAfterConversion.
        ].

convBm add: self.
^true

]

{ #category : 'AST-Kernel-Core' }
Object >> value [
  "in bootstrap files in case upgrade stops after base image bootstrap"
  ^self

]
