Extension { #name : 'JISString' }

{ #category : 'Formatting' }
JISString class >> charSize [

"Returns the number of bytes that make up a character"
^2

]

{ #category : 'Indexing Support' }
JISString >> _isValidPathTermName [

"Returns true if the receiver is a valid term in a path expression."

self deprecated:'_isValidPathTermName deprecated (JISString not supported path terms)'.
^ false

]

{ #category : 'Concatenating' }
JISString >> , aCharOrCharColl [

"Returns a new instance of the receiver's class that contains the elements of
 the receiver followed by the elements of aCharOrCharColl.

 Note: Creating a new instance and copying the receiver take time.  If you can
 safely modify the receiver, it is faster to use the addAll: method."
 | res  |
 (res := JISString new) addAll: self;
               addAll: aCharOrCharColl.
 ^ res

]

{ #category : 'Adding' }
JISString >> add: aCharOrCharCollection [

"Appends aCharOrCharCollection to the receiver.  The argument
 aCharOrCharCollection must be a CharacterCollection or a kind of
 AbstractCharacter.
 Returns aCharOrCharCollection "

(aCharOrCharCollection isKindOf: Collection) ifTrue:[
   self addAll: aCharOrCharCollection.
   ^ aCharOrCharCollection
 ].
self at: (self size + 1) put: aCharOrCharCollection .
^ aCharOrCharCollection

]

{ #category : 'Adding' }
JISString >> addAll: aCharOrCharCollection [

"Equivalent to add: aCharOrCharCollection."

| idx |

(aCharOrCharCollection isKindOf: Collection) ifTrue:[
  idx := self size .
  aCharOrCharCollection do:[ :aChar |
    idx := idx + 1 .
    self at: idx put: aChar .
    ].
  ^ aCharOrCharCollection
  ].
^ self at: (self size + 1) put: aCharOrCharCollection

]

{ #category : 'Adding' }
JISString >> addLast: aCharOrCharCollection [

"Equivalent to add: aCharOrCharCollection."

^ self add: aCharOrCharCollection

]

{ #category : 'Formatting' }
JISString >> asJISString [

"Returns the receiver."

"This reimplementation from JapaneseString is crucial!"

^self

]

{ #category : 'Converting' }
JISString >> asSymbol [

"Returns the receiver as a DoubleByteSymbol."

^ (DoubleByteString withAll: self) asSymbol

]

{ #category : 'Converting' }
JISString >> asSymbolKind [

"Returns a copy of the receiver as an instance of class Symbol."

^ self asSymbol


]

{ #category : 'Accessing' }
JISString >> at: anIndex [

"Returns the JISCharacter at anIndex."

| highByte lowByte offset |

offset := anIndex + anIndex  .
lowByte := self _basicAt: offset .
highByte := self _basicAt: offset - 1 .
 ^ JISCharacter withValue: highByte * 256 + lowByte

]

{ #category : 'Updating' }
JISString >> at: anIndex put: aCharacter [

"Stores aCharacter at the specified location."

| offset val highByte lowByte |
offset := anIndex + anIndex  .
val := aCharacter asJISCharacter jisValue .
highByte := val // 256 .
lowByte :=  val \\ 256 .
self _basicAt: offset - 1 put: highByte .
self _basicAt: offset     put: lowByte .
^ aCharacter

]

{ #category : 'Formatting' }
JISString >> charSize [

"Returns the number of bytes that make up a character"
^2

]

{ #category : 'Accessing' }
JISString >> codePointAt: anIndex [

^ (self at: anIndex) jisValue

]

{ #category : 'Updating' }
JISString >> codePointAt: anIndex put: aValue [

^ self at: anIndex put:(JISCharacter withValue: aValue)

]

{ #category : 'Searching' }
JISString >> indexOf: aCharacter startingAt: startIndex [

"Returns the index of the first occurrence of aCharacter in the receiver,
 not preceding startIndex.  If the receiver does not contain aCharacter,
 this returns zero."

startIndex to: self size do:[:j|
  (self at:j ) == aCharacter ifTrue:[ ^ j ].
  ].
^ 0

]

{ #category : 'Adding' }
JISString >> insertAll: aCharOrCharCollection at: anIndex [

"Inserts aCharOrCharCollection at the specified index."

<primitive: 241>

anIndex _validateClass: SmallInteger.
(anIndex > (self size + 1))
ifTrue:[ ^ self _errorIndexOutOfRange: anIndex]
ifFalse:[ ^ super insertAll: aCharOrCharCollection at: anIndex]

]

{ #category : 'Accessing' }
JISString >> size [

"Returns the size of the receiver in Characters."

^ self _basicSize // 2

]

{ #category : 'Updating' }
JISString >> size: anInteger [

"Changes the size of the receiver to anInteger.

 If anInteger is less than the current size of the receiver, the receiver is
 shrunk accordingly.  If anInteger is greater than the current size of the
 receiver, the receiver is extended and new elements are initialized to nil."

^ self _basicSize: anInteger + anInteger

]
